/** Ludovic Apvrille

ludovic.apvrille AT enst.fr

This software is a computer program whose purpose is to allow the 
edition of Latex documents

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

/**
 * Class Main
 * Starts the main Windows and a project manager
 * Creation: 11/06/2010
 * @version 1.0 11/06/2009
 * @author Ludovic APVRILLE
 * @see
 */

import java.awt.*;
import java.awt.event.*;
import java.util.Locale;
import java.net.URL;
import java.util.*;
//import java.io.*;
import javax.swing.ImageIcon;
import javax.swing.*;

import ui.*;
import ui.window.*;
import myutil.*;
import template.*;

public class Main implements ActionListener {
    public boolean finish = false;
    public static Main main = new Main();
    public static void main(String[] args) {
      
        TraceManager.addUser("\n*** Your Latex Editor version is: " + DefaultText.getFullVersion() + " ***\n");
		
        // Read the image data and display the splash screen
        // -------------------------------------------------
        TraceManager.devPolicy = TraceManager.TO_DEVNULL;
        JStartingWindow splashFrame = null;
		MyDictionary dic = null;
		
        //System.out.println("hello");
        URL imageURL = Main.class.getResource("ui/images/starting_logo.gif");
        Image img;
        //System.out.println("helli");
        if (imageURL != null) {
            //System.out.println("hellu");
            img = (new ImageIcon(imageURL)).getImage();
            splashFrame = JStartingWindow.splash(img, "Loading LatexEditor's elements");
            //splashFrame.setLicenceMessage("An open-source toolkit from:");
            splashFrame.setMiddleMessage("version " + DefaultText.getVersion());
            //System.out.println("helly");
        } else {
            TraceManager.addError("Starting image not found");
            System.exit(0);
        }
        
        
        new javax.swing.Timer(2000, main).start();
        
        
        
        // Starting window
        // setting default language
        splashFrame.setMessage("Setting language");
        Locale.setDefault(new Locale("en"));
        
        // Analyzing arguments
        String config = "config.xml";
        for(int i=0; i<args.length; i++) {
            
            if (args[i].compareTo("-config") == 0) {
                config = args[Math.min(args.length-1, i+1)];
            }
			
			if (args[i].compareTo("-debug") == 0) {
                TraceManager.devPolicy = TraceManager.TO_CONSOLE;
            }
        }
        
        // Icons
        splashFrame.setMessage("Loading images");
        IconManager icma = new IconManager();
        icma.loadImg();
        
        // Loading configuration
        splashFrame.setMessage("Loading configuration file: " + config);
        
        
        try {
            ConfigurationLatexEditor.loadConfiguration(config);
        } catch (Exception e) {
            TraceManager.addDev("Couldn't load configuration properly : " + e.toString());
        }
        TraceManager.addDev("\nConfiguration:\n--------------");
        ConfigurationLatexEditor.printConfiguration();
        TraceManager.addDev("\nDebugging trace:\n----------------");
        
        
		
        // making main window
        splashFrame.setMessage("Creating main window");
        MainGUI mainGUI = new MainGUI();
       
		// Loading Dictionaries
		splashFrame.setMessage("Loading dictionaries");
		String tmp;
		boolean b;
		if ((ConfigurationLatexEditor.DataPath != null) && (ConfigurationLatexEditor.Dictionary != null)) {
			String dics[] = ConfigurationLatexEditor.Dictionary.split(" ");
			for (int i=0; i<dics.length; i++) {
				if (dics[i].trim().length() > 0) {
					tmp = dics[i].trim();
					if (tmp.charAt(0) == '*') {
						tmp = tmp.substring(1, tmp.length());
						dic = new MyDictionary(tmp);
						dic.setSelected(true);
					} else {
						dic = new MyDictionary(tmp);
						dic.setSelected(false);
					}
					int n = dic.loadFromFile(ConfigurationLatexEditor.DataPath + tmp);
					if (n > -1) {
						TraceManager.addUser("Dictionary loaded from " + ConfigurationLatexEditor.DataPath + dics[i].trim() + ": " + n + " word");
						mainGUI.addDictionary(dic);
					} else {
						TraceManager.addUser("Error when loading dictionary");
					}
				}
			}
		}
		if ((ConfigurationLatexEditor.DataPath != null) && (ConfigurationLatexEditor.PersonalDictionary != null)) {
			String persdic = ConfigurationLatexEditor.PersonalDictionary.trim();
			if (persdic.length() > 0) {
				if (persdic.charAt(0) == '*') {
					persdic = persdic.substring(1, persdic.length());
					dic = new MyDictionary(persdic);
					dic.setSelected(true);
				} else {
					dic = new MyDictionary(persdic);
					dic.setSelected(false);
				}
				int n = dic.loadFromFile(ConfigurationLatexEditor.DataPath + persdic);
				if (n > -1) {
					TraceManager.addUser("Dictionary loaded from " + ConfigurationLatexEditor.DataPath + persdic + ": " + n + " word");
					mainGUI.addDictionary(dic);
				} else {
					TraceManager.addUser("Error when loading dictionary");
				}
			}
		}
		
		splashFrame.setMessage("Loading templates");
		LinkedList<Template> templates = null;
		if ((ConfigurationLatexEditor.DataPath != null) && (ConfigurationLatexEditor.TemplatesPath != null) && (ConfigurationLatexEditor.TemplatesPath.length() > 0)) {
			templates = Template.loadTemplatesFromFile(ConfigurationLatexEditor.DataPath + "/" + ConfigurationLatexEditor.TemplatesPath);
		}
		TraceManager.addUser("" + templates.size() + " template(s) have been loaded");
		mainGUI.setTemplates(templates);
        
        // starting application
        if (mainGUI == null) {
            ErrorGUI.exit(ErrorGUI.GUI);
        }
		
		splashFrame.setMessage("Building graphical components");
		mainGUI.build();
		
        
        splashFrame.setMessage("Starting LOBSTER (Latex and Beamer Editor) ...");
        main.waitFinish();
		
        mainGUI.start();
        
        // Dispose the splash screen
        // -------------------------
        if (splashFrame != null) {
            splashFrame.dispose();
        }
        
    }
    
    public synchronized void waitFinish() {
        while(finish == false) {
            try {
                wait();
            } catch (InterruptedException ie) {
            }
        }
    }
    
    public synchronized void okFinish() {
        finish = true;
        notifyAll();
    }
    
    public void actionPerformed(ActionEvent evt) {
        main.okFinish();
    }
    
} // Class Main

