/**Copyright or (C) or Copr. GET / ENST, Telecom-Paris, Ludovic Apvrille

ludovic.apvrille AT enst.fr

This software is a computer program whose purpose is to allow the
edition of Latex documents

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

/**
* Class LatexStructure
* Creation: 11/06/2010
* version 1.0 11/06/2010
* @author Ludovic APVRILLE
* @see 
*/

package latexstructure;

import java.io.*;
import java.util.*;

import myutil.*;


public	class LatexStructure implements GenericTree {
	
	public static final String LATEX_BEGIN_DOC = "\\begin{document}";
	public static final String LATEX_END_DOC = "\\end{document}";

	public static final String LATEX_FIRST = "%Generated by Lobster\n\n\\documentclass[11pt,a4paper]{article}\n\n\\begin{document}";
	public static final String LATEX_LAST = "\\end{document}";
	
	public static final String CR = "\n";
	public static final String CR2 = CR + CR;
	
	private static final String[] TO_KEEP_IN_NOTES = {"\\section", "\\subsection", "\\subsubsection", "\\subsubsubsection", "\\frametitle"};
	private static final String[] TO_KEEP_IN_NOTES_MODIFIED = {"\\section", "\\subsection", "\\subsubsection", "\\subsubsubsection", "\\subsubsection"};
	
	private static final String[] NOTES = {"%note:", "%Note:"};
	
	public static boolean showLabels = true;
	public static boolean showOthers = true;
	public static boolean showSlides = true;
	public static boolean showSections = true;
	
	
	private String data;
	
	private String name;
	
	private LinkedList<LatexElement> elements;
	
	private boolean toUpdate;
	
	
	public LatexStructure(String _data) {
		data = _data;
		elements = new LinkedList<LatexElement>();
	}
	
	public void setData(String _data) throws MalformedLatexDocumentException {
		data = _data;
	}
	
	public String toString() {
		return name;
	}
	
	public void setName(String _name) {
		name = _name;
	}
	
	public boolean hasText() {
		return (data != null);
	}
	
	public String getText() {
		return data;
	}
	
	public void setText(String _data) {
		data = _data;
	}
	
	public void setUpdate() {
		toUpdate = true;
	}
	
	public boolean hasBeenUpdated() {
		return toUpdate;
	}
	
	public void makeStructure(boolean _flatStructure) {
		if (!toUpdate) {
			return;
		}
		toUpdate = false;
		
		LinkedList<LatexElement> newSyst = new LinkedList<LatexElement>();
		
		String input = data;
		int totalIndex = 0;
		int localIndex[];
		LatexElement le1, le2 = null;
		int level;
		int indexc;
		boolean comment;
		int i;
		
		// We've got to ignore what is commented
		while((localIndex = findKeyword(input))[0] != -1) {
			//TraceManager.addDev("Found keyword:" + LatexElement.keywords[localIndex[1]]);
			if (LatexElement.levels[localIndex[1]] == -1) {
				// Comment?
				comment = false;
				if (localIndex[0] == 0) {
					comment = true;
				} else {
					comment = (!(input.substring(localIndex[0]-1, localIndex[0]).compareTo("\\") == 0));
				}
				//TraceManager.addDev("Comment found?:" + comment + " in " + input);
				input = input.substring(localIndex[0]+1, input.length());
				totalIndex = totalIndex + localIndex[0] + 1;
				if (comment) {
					//TraceManager.addDev("Found comment in:" + input);
					indexc = input.indexOf('\n');
					if (indexc == -1) {
						input = "";
					} else {
						input = input.substring(indexc+1, input.length());
						totalIndex = totalIndex + indexc + 1;
					}
					//TraceManager.addDev("After removing comment:" + input);
				} 
				//TraceManager.addDev("Comment. TotalIndex:" + totalIndex);
			} else {
				totalIndex = totalIndex + localIndex[0] + 1;
				if (le2 == null) {
					le2 = new LatexElement(localIndex[1], LatexElement.getName(input, localIndex[0], localIndex[1]), totalIndex, null);
					newSyst.add(le2);
				} else {
					level = LatexElement.levels[localIndex[1]];
					while(le2.getLevel() >= level) {
						le2 = le2.getFather();
						if (le2 == null) {
							break;
						}
					}
					
					if ((_flatStructure) || (le2 == null)){
						le2 = new LatexElement(localIndex[1], LatexElement.getName(input, localIndex[0], localIndex[1]), totalIndex, null);
						newSyst.add(le2);
					} else {
							if (LatexElement.levels[le2.getType()] > 0) {
							le1 = new LatexElement(localIndex[1], LatexElement.getName(input, localIndex[0], localIndex[1]), totalIndex, le2);
							le2.addElement(le1);
							le2 = le1;
						} else {
							le2 = new LatexElement(localIndex[1], LatexElement.getName(input, localIndex[0], localIndex[1]), totalIndex, null);
							newSyst.add(le2);
						}
					}
					
				}
				//TraceManager.addDev("Regular. TotalIndex:" + totalIndex);
				input = input.substring(localIndex[0]+1, input.length());
				    
			}
		}
		//elements = null;
		if (elements == null) {
			elements = newSyst;
			return;
		}
		
		// Same structure begin -> index until which the system is the same
		// Same structure end -> index after which the system is the same
		int begin, end;
		boolean failure = false;
		int nb;
		
		for(begin=0; begin<Math.min(elements.size(), newSyst.size()); begin++) {
			if (!(elements.get(begin).equals(newSyst.get(begin)))) {
				//TraceManager.addDev("Difference: elt:" + elements.get(begin) + " new:" + newSyst.get(begin));
				failure = true;
				break;
			}
		}
		
		if (!failure) {
			TraceManager.addDev("No failure");
			// Lists are similar?
			// If of the same size: yes!
			
			if (newSyst.size() > elements.size()) {
				// Must add new elements
				TraceManager.addDev("New elements");
				for(i=begin; i<newSyst.size(); i++) {
					elements.add(newSyst.get(i));
				}
			} else if (newSyst.size() < elements.size()) {
				// Must remove elements from elements
				TraceManager.addDev("Elements to remove");
				nb = elements.size() - newSyst.size();
				for(i=0; i<nb; i++) {
					elements.removeLast();
				}
			}
			
			// List are now of the same size!
			updateIndexes(elements, newSyst);
			
			return;
		}
		
		TraceManager.addDev("Failure begin=" + begin);
		// So the element at index begin is different
		// Must find the end index
		for(end=0; end < Math.min(elements.size(), newSyst.size()); end++) {
			if (!(elements.get(elements.size()-end-1).equals(newSyst.get(newSyst.size()-end-1)))) {
				break;
			}
		}
		
		TraceManager.addDev("Failure begin=" + begin + " end=" + end);
		// We remove elements from begin to end into elements
		nb = elements.size() - (begin + end);
		TraceManager.addDev("Removing " + nb + " elements");
		for(i=0; i<nb; i++) {
			elements.remove(begin);
		}
		
		// We insert elements from begin to end into elements
		nb = newSyst.size() - (begin + end);
		TraceManager.addDev("Adding " + nb + " elements");
		for(i=nb-1; i>-1; i--) {
			elements.add(begin, newSyst.get(begin+i));
		}
		
		updateIndexes(elements, newSyst);
		
	}
	
	private void updateIndexes(LinkedList<LatexElement> _toUpdate, LinkedList<LatexElement> _list) {
		for(int i=0; i <Math.min(_toUpdate.size(), _list.size()); i++) {
			_toUpdate.get(i).updateIndexes(_list.get(i));
		}
	}

	
	// Returns [index ds le tex, index du keyword]
	public static int[] findKeyword(String tmp) {
		int [] infos = new int[2];
		infos[0] = -1;
		int indextmp;
		for(int i=0; i<LatexElement.keywords.length; i++) {
			indextmp = tmp.indexOf(LatexElement.keywords[i]);
			if (indextmp != -1) {
				if (infos[0] == -1) {
					infos[0] = indextmp;
					infos[1] = i;
				} else {
					infos[0] = Math.min(infos[0], indextmp);
					if (infos[0] == indextmp) {
						infos[1] = i;
					}
				}
			}
		}
		return infos;
		
	}
	
	public static int findClosingBracket(String s) {
		int nbFound = 0;
		int cpt = 0;
		
		while(cpt<s.length()) {
			if (s.charAt(cpt) == '\\') {
				cpt ++;
			} else {
				if (s.charAt(cpt) == '}') {
					if (nbFound == 0) {
						return -1;
					} else if (nbFound == 1) {
						return cpt;
					} else {
						nbFound --;
					} 
				} else if (s.charAt(cpt) == '{') {
					nbFound ++;
				}
			}
			cpt ++;
		}
		return -1;
	}
	
	public LatexStructure getACloneOfMe() throws CloneNotSupportedException {
		LatexStructure ls = new LatexStructure(data);
		try {
			ls.setName(getName());
		} catch (Exception e) {
			throw new CloneNotSupportedException(e.getMessage());
		}
		return ls;
	}
	
	public int getChildCount() {
		int cpt = 0;
		for(LatexElement le: elements) {
			if (le.isOther() && showOthers) {
				cpt ++;
			} else if (le.isLabel() && showLabels){
				cpt ++;
			}  else if (le.isSlide() && showSlides){
				cpt ++;
			}  else if (le.isSection() && showSections){
				cpt ++;
			}
		}
		
		return cpt;
	}
	
    public Object getChild(int _index) {
		int cpt = 0;
		for(LatexElement le: elements) {
			if (le.isOther() && showOthers) {
				if (cpt == _index) {
					return le;
				}
				cpt ++;
			} else if (le.isLabel() && showLabels){
				if (cpt == _index) {
					return le;
				}
				cpt ++;
			} else if (le.isSlide() && showSlides){
				if (cpt == _index) {
					return le;
				}
				cpt ++;
			}  else if (le.isSection() && showSections){
				if (cpt == _index) {
					return le;
				}
				cpt ++;
			}
		}
		
		return null;
	}
	
	public int getIndexOfChild(Object _child) {
		int cpt =0;
		for(LatexElement le: elements) {
			if (le.isOther() && showOthers) {
				if (le == _child) {
					return cpt;
				}
				cpt ++;
			} else if (le.isLabel() && showLabels){
				if (le == _child) {
					return cpt;
				}
				cpt ++;
			} else if (le.isSlide() && showSlides){
				if (le == _child) {
					return cpt;
				}
				cpt ++;
			}  else if (le.isSection() && showSections){
				if (le == _child) {
					return cpt;
				}
				cpt ++;
			}
		}
		
		return -1;
	}
	
	public String getName() {
		return name;
	}
	
	public String print(int _index) {
		return data.substring(Math.max(0, _index), Math.min(data.length(), _index+10));
	}
	
	public Object[] findPath(Object[] _path) {
		TraceManager.addDev("findPath");
		Object[] objs = new Object[_path.length];
		
		objs[0] = _path[0];
		
		for(LatexElement le: elements) {
			TraceManager.addDev("In: " + le.getName());
			if (le.getPath(objs, _path, 1)) {
				return objs;
			}
			TraceManager.addDev("did not worked in: " + le.getName());
		}
		
		return null;
	}
	
	// Management of notes
	public synchronized String getNotes(String _title) {
		StringBuffer sb = new StringBuffer("");
		sb.append(LATEX_FIRST + CR2);
		
		sb.append("\\title{Notes on presentation " + _title + "}" + CR);
		sb.append("\\date{\\today}" + CR2);
		
		sb.append("\\maketitle" + CR2);
		
		Scanner scanner = new Scanner(data);
		int i;
		while (scanner.hasNextLine()) {
			String line = scanner.nextLine().trim();
			for(i=0; i<TO_KEEP_IN_NOTES.length; i++) {
				if (line.startsWith(TO_KEEP_IN_NOTES[i])) {
					line = TO_KEEP_IN_NOTES_MODIFIED[i] + line.substring((TO_KEEP_IN_NOTES[i].length()), line.length());
					sb.append(line + CR2);
				}
			}
			for(i=0; i<NOTES.length; i++) {
				if (line.startsWith(NOTES[i])) {
					sb.append(line.substring(NOTES[i].length(), line.length()) + CR2);
				}
			}
			
		}
		sb.append(LATEX_LAST + CR);
		return sb.toString();
	}
	
	public synchronized String getCurrent(int _indexOfLine, String _currentText, String _title) {
		StringBuffer sb = new StringBuffer("");
		
		int index0 = data.indexOf("\\begin{document}");
		int index1 = _currentText.indexOf("\\end{document}");
		
		if (index0 == -1) {
			return _currentText;
		}
		
		sb.append(data.substring(0, index0));
		sb.append("\\begin{document}\n");
		
		Scanner scanner = new Scanner(data.substring(index0+1, Math.min(_indexOfLine, data.length())));
		while (scanner.hasNextLine()) {
			String line = scanner.nextLine().trim();
			if (line.startsWith("\\setbeamer")) {
				sb.append(line);
			} else if (line.startsWith("\\section")) {
				sb.append(line);
			}else if (line.startsWith("\\subsection")) {
				sb.append(line);
			}
		}
		
		sb.append(_currentText);
		
		if (index1 == -1) {
			sb.append("\\end{document}");
		}
		return sb.toString();
	}
	
	// Management of current view
	// Return the first and the last index in the main document
	// of the subelement in which is the _currentPositionInText
	// A subelemnt stops when an element of the same hierarchical level is met
	public int[] getCurrentIndexesOfMainPosition(int _currentPositionInText) {
		_currentPositionInText = Math.max(0, _currentPositionInText);

		int tab[] = new int[2];
		
		if (elements.size() ==0) {
			tab[0] = 0;
			tab[1] = data.length();
			return tab;
		}
		
		if (elements.get(0).getIndex() > _currentPositionInText) {
			tab[0] = 0;
			tab[1] = elements.get(0).getIndex();
			return tab;
		}
		
		// We search for the first element before 
		LatexElement foundLe = null;
		LatexElement le;
		for(int i=elements.size()-1; i>-1; i--) {
			le = elements.get(i);
			foundLe = le.getJustBeforeIndex(_currentPositionInText);
			if (foundLe != null) {
				break;
			}
		}
		
		if (foundLe == null) {
			tab[0] = 0;
		} else {
			tab[0] = foundLe.getIndex();
		}
		
		// Must locate the element right after that position
		LatexElement foundLe2 = null;
		for(LatexElement le2: elements) {
			foundLe2 = le2.getAfterIndex(_currentPositionInText);
			if (foundLe2 != null) {
				break;
			}
		}
		
		if (foundLe2 == null) {
			tab[1] = data.length();
		} else {
			tab[1] = foundLe2.getIndex();
		}
		
		return tab;
		
	}
	
	public LatexElement getVeryLastLatexElement() {
		return elements.get(elements.size()-1).getVeryLastLatexElement();
	}
	
	
	
} // Class 

