/**Copyright or (C) or Copr. GET / ENST, Telecom-Paris, Ludovic Apvrille

ludovic.apvrille AT enst.fr

This software is a computer program whose purpose is to allow the
edition of Latex documents

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

/**
* Class JPanelForCompilation
* Creation: 11/06/2010
* Version 1.0 11/06/2010
* @author Ludovic APVRILLE
* @see
*/

package ui;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import latexstructure.*;
import myutil.*;


public	class JPanelForCompilation extends JPanel implements ActionListener, Runnable, Stoppable {
	
    private boolean compilationRunning;
	private MainGUI mgui;
	private Thread compilationThread;
	private String compileCommand;
	
	protected JButton stop;
	protected JTextArea jta;
	
	protected Process proc;
	protected BufferedReader proc_in;
	protected BufferedWriter proc_out;
	
	protected boolean errorFound;
	protected String errorInfo;
	protected String errorLine;
	
	protected JTextField command;
	protected JButton goCommand;
	
    
    public JPanelForCompilation(MainGUI _mgui) {
        super();
        mgui = _mgui;
		
		makeComponents();
		setComponents();
		
    }
	
	
	public void makeComponents() {
		
		JPanel panel1 = new JPanel();
		panel1.add(new JLabel("Command:"));
		command = new JTextField("My command", 30);
		panel1.add(command);
		goCommand = new JButton("Run command", IconManager.imgic34);
		goCommand.addActionListener(this);
		panel1.add(goCommand);
		
		setLayout(new BorderLayout());
		
		jta = new ScrolledJTextArea();
		jta.setEditable(false);
		jta.setMargin(new Insets(10, 10, 10, 10));
		jta.setTabSize(3);
		jta.append("Compilation window\n");
		Font f = new Font("Courrier", Font.BOLD, 10);
		jta.setFont(f);
		JScrollPane jsp = new JScrollPane(jta, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		
		add(panel1, BorderLayout.NORTH);
		add(jsp, BorderLayout.CENTER);
		
		stop = new JButton("Stop", IconManager.imgic27);
		stop.setPreferredSize(new Dimension(200, 30));
		stop.addActionListener(this);
		add(stop, BorderLayout.SOUTH);
	}
	
	public void setComponents() {
		stop.setEnabled(false);
	}
	
	public void compile(String _compileCommand) {
		if (compilationThread == null) {
			mgui.setMode(MainGUI.COMPILING);
			compileCommand = _compileCommand;
			stop.setEnabled(true);
			compilationThread = new Thread(this);
			compilationThread.start();
		}
	}
	
	/*public void execute(String _command) {
	try {
	proc = Runtime.getRuntime().exec(_command);
	} catch (Exception e) {
	TraceManager.addDev("Exception when starting command: " + _command + " :" + e.getMessage());
	}
	}*/
	
	public void run() {
		String line;
		errorFound = false;
		errorLine = null;
		
		jta.append("Running command:" + compileCommand + "\n");
		
		try {
			proc = Runtime.getRuntime().exec(compileCommand);
			proc_in = new BufferedReader(new InputStreamReader(proc.getInputStream()));
			
			// Tries to send a "return" to the process
			proc_out = new BufferedWriter(new OutputStreamWriter(proc.getOutputStream()));
			proc_out.write("q\n", 0, 2);
			proc_out.flush();
			//System.out.println("q sent");
			
			while (((line = proc_in.readLine()) != null) && (stop.isEnabled() == true)){
				jta.append("->" + line + "\n");
				if (line.trim().length() > 0) {
					if (line.trim().charAt(0) == '!') {
						errorFound = true;
						errorInfo = line.substring(1, line.length());
					}
					if (line.trim().startsWith("l.") && errorFound) {
						String tmp = line.trim().substring(2, line.length());
						errorLine = "";
						for(int i=0; i<tmp.length(); i++) {
							if (Character.isDigit(tmp.charAt(i))) {
								errorLine = errorLine + tmp.charAt(i);
							} else {
								break;
							}
						}
					}
				}
			}
			
			//proc = null;
			//jta.append("Command executed\n");
		} catch (Exception e) {
			jta.append("Exception: " + e.getMessage() + "\n");
		}
		
		try {
			proc_in.close();
		} catch (Exception e) {
		}
		
		try {
			proc_out.close();
		} catch (Exception e) {
		}
		
		try {
			proc.destroy();
		} catch (Exception e) {
		}
		
		compilationEnded();
	}
	
	public synchronized void compilationEnded() {
		proc = null;
		if (stop.isEnabled() == false) {
			jta.append("** Process killed **\n");
		} else {
			jta.append("** All done **\n");
			stop.setEnabled(false);
		}
		
		if(errorFound) {
			if (errorLine != null) {
				jta.append("\n*ERROR* on line " + errorLine + ":" + errorInfo + "\n\n");
			} else {
				jta.append("\n*ERROR*:" + errorInfo + "\n\n");
			}
			
		}
		compilationThread = null;
		mgui.setMode(MainGUI.COMPILING_DONE);
	}
	
	public synchronized void stopMe() {
		if (proc != null) {
			try {
				proc_in.close();
			} catch (Exception e) {
			}
			
			try {
				proc_out.close();
			} catch (Exception e) {
			}
			
			try {
				proc.destroy();
			} catch (Exception e) {
			}
		}
		stop.setEnabled(false);
	}
	
	public void	actionPerformed(ActionEvent evt)  {
        
        // Compare the action command to the known actions.
		if (evt.getSource() == stop)  {
			new ThreadToStop(this);
		} else if (evt.getSource() == goCommand) {
				compile(command.getText());
		}
    }
	
	
	
} // Class

